@@ -20,6 +20,7 @@ import (
 	"testing"
 
 	"cuelang.org/go/cue/ast"
+	"cuelang.org/go/internal/astinternal"
 )
 
 func TestParse(t *testing.T) {
@@ -80,6 +81,30 @@ func TestParse(t *testing.T) {
 		`,
 		`if=foo: 0, for=bar: 2, let=bar: 3, func=baz: 4`,
 	}, {
+		"keywords as selector",
+		`a : {
+			if: 0
+			for: 1
+			in: 2
+			where: 3
+			div: 4
+			quo: 5
+			func: 6
+			float: 7
+			null: if: func: let: 3
+		}, b: [
+			a.if,
+			a.for,
+			a.in,
+			a.where,
+			a.div,
+			a.quo,
+			a.func,
+			a.float,
+			a.null.if.func.let,
+		]`,
+		`a: {if: 0, for: 1, in: 2, where: 3, div: 4, quo: 5, func: 6, float: 7, null: {if: {func: {let: 3}}}}, b: [a.if, a.for, a.in, a.where, a.div, a.quo, a.func, a.float, a.null.if.func.let]`,
+	}, {}, {
 		"json",
 		`{
 			"a": 1,
@@ -235,6 +260,11 @@ func TestParse(t *testing.T) {
 			[X=_]: X
 		`,
 		"a: {b: {c: d}}, c: a, d: a.b, e: a.b.c, \"f\": f, [X=_]: X",
+	}, {
+		"predeclared identifiers",
+		`a:    __string
+		__int: 2`,
+		"a: __string, __int: 2\nidentifiers starting with '__' are reserved",
 	}, {
 		"empty fields",
 		`
@@ -685,7 +715,7 @@ bar: 2
 				mode = append(mode, ParseFuncs)
 			}
 			f, err := ParseFile("input", tc.in, mode...)
-			got := debugStr(f)
+			got := astinternal.DebugStr(f)
 			if err != nil {
 				got += "\n" + err.Error()
 			}
@@ -858,7 +888,7 @@ func TestIncompleteSelection(t *testing.T) {
 				return true
 			}, nil)
 			if sel == nil {
-				t.Fatalf("found no *SelectorExpr: %#v %s", f.Decls[0], debugStr(f))
+				t.Fatalf("found no *SelectorExpr: %#v %s", f.Decls[0], astinternal.DebugStr(f))
 			}
 			const wantSel = "&{fmt _ {<nil>} {{}}}"
 			if fmt.Sprint(sel) != wantSel {
@@ -877,5 +907,5 @@ func TestX(t *testing.T) {
 	if err != nil {
 		t.Errorf("unexpected error: %v", err)
 	}
-	t.Error(debugStr(f))
+	t.Error(astinternal.DebugStr(f))
 }
